LOGIN_POST_URL = "https://jaccount.sjtu.edu.cn/jaccount/ulogin"
CAPTCHA_URL = "https://jaccount.sjtu.edu.cn/jaccount/captcha"

DEFAULT_BASE_URL = "https://i.sjtu.edu.cn"

HOME_URL = "/xtgl/index_initMenu.html"
SCHEDULE_URL = "/kbcx/xskbcx_cxXsKb.html?gnmkdm=N2151"
LOGIN_URL = "/jaccountlogin"
LOGOUT_URL = "/logout"
SCORE_URL = "/cjcx/cjcx_cxDgXscj.html?doType=query&gnmkdm=N305005"
SCORE_DETAIL_URL = "/cjcx/cjcx_cxXsXmcjList.html?gnmkdm=N305007&su="
EXAM_URL = "/kwgl/kscx_cxXsksxxIndex.html?doType=query&gnmkdm=N358105&su="
CALENDAR_URL = "/xtgl/index_cxshjdAreaFive.html?localeKey=zh_CN&gnmkdm=index&su="
COURSELIB_URL = "/design/funcData_cxFuncDataList.html?func_widget_guid=DA1B5BB30E1F4CB99D1F6F526537777B&gnmkdm" \
                "=N219904&su= "
GPA_PARAMS_URL = "/cjgl/common_cxGnzdxxList.html?gnmkdm=N309130"
GPA_CALC_URL = "/cjpmtj/gpapmtj_tjGpapmtj.html?gnmkdm=N309131&su="
GPA_QUERY_URL = "/cjpmtj/gpapmtj_cxGpaxjfcxIndex.html?doType=query&gnmkdm=N309131&su="
PROFILE_URL = "/xsxxxggl/xsgrxxwh_cxXsgrxx.html?gnmkdm=N100801&layout=default&su="
SELECTION_ALL_SECTORS_PARAM_URL = "/xsxk/zzxkyzb_cxZzxkYzbIndex.html?gnmkdm=N253512&layout=default&su="
SELECTION_SECTOR_PARAM_URL = "/xsxk/zzxkyzb_cxZzxkYzbDisplay.html?gnmkdm=N253512&su="
SELECTION_QUERY_COURSES = "/xsxk/zzxkyzb_cxZzxkYzbPartDisplay.html?gnmkdm=N253512&su="
SELECTION_QUERY_CLASSES = "/xsxk/zzxkyzb_cxJxbWithKchZzxkYzb.html?gnmkdm=N253512&su="
SELECTION_REGISTER = "/xsxk/zzxkyzb_xkBcZyZzxkYzb.html?gnmkdm=N253512&su="
SELECTION_IS_REGISTERED = "/xsxk/zzxkyzb_xkJcInXksjZzxkYzb.html?gnmkdm=N253512&su="
SELECTION_DEREGISTER = "/xsxk/zzxkyzb_tuikBcZzxkYzb.html?gnmkdm=N253512&su="

HEADERS = {"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:71.0) Gecko/20100101 Firefox/71.0",
           "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8",
           "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7,zh-TW;q=0.6"}
TERMS = [3, 12, 16]
CHINESE_WEEK = {"日": 0, "一": 1, "二": 2, "三": 3, "四": 4, "五": 5, "六": 6}
