from pysjtu.ocr import LegacyRecognizer, NNRecognizer, JCSSRecognizer
from .client import Client, create_client
from .models import CourseRange, LogicEnum, Ranking
from .session import Session

__version__ = "0.3.4"
